/*
	Word Twiddle
	JavaScript game engine
	Version 0.9
	Copyright 2011 by Andrew Krywaniuk
	
	License is hereby granted to freely distribute this game for personal and non-commercial use.
	For other uses, please contact the author.
*/


// total_word_lists is an array of arrays for each word length (using 1-based indexing).
var total_word_lists = [ null, null, null, null, null, null, null, null];

// word map is a flat list of all words indexed by hash value.
var total_word_map = new Object();


// letter vals - used in the hash function.
var letter_vals = [
	0x00100101, 0x000d0000, 0x00800c00, 0x00300010, 0x02004010, 0x000400b0, 0x00006000,  // abcdefg
	0x00000201, 0x0000d010, 0x000000c0, 0x00030100, 0x00040010, 0x00000130, 0x00005010,  // hijklmn
	0x00301010, 0x00000e04, 0x00000120, 0x000030c0, 0x00404001, 0x00080120, 0x00015030,  // opqrstu
	0x00060020, 0x00040d20, 0x00080506, 0x00003107, 0x0000010c  // vwxyz
	];
	
	
function hash_string(str)
{
	var n = str.length;
	var i;
	var ch;
	var hash = 0;
	
	for (i=0; i<n; i++)
	{
		ch = str.charCodeAt(i);
		idx = ch - "a".charCodeAt(0);
		
		// Check for illegal characters in string.
		if (idx < 0 || idx >= 26) return 0;
		
		hash += letter_vals[idx];
	}
	
	return hash;
}


function load_wt_data(word_len)
{
	var frm_name = "w" + word_len;
	var ofrm = document.getElementById(frm_name);
	var data = ofrm.contentDocument.body.innerHTML
	var len = data.length;
	
	// Create a new word list for this word length.
	var aWords = []; 
	
	
	// strip the <pre> & </pre>
	data = data.substr(5, len - 11);
	
	var i;
	var j;
	
	var a_lines = data.split("\n");
	var n = a_lines.length;
	
	for (i=0; i<n; i++)
	{
		var a_words = a_lines[i].split(" ");
		var m = a_words.length;
		
		for (j=0; j<m; j++)
		{
			var wd = a_words[j];
			if (wd.length == 0) continue;
			
			aWords.push(wd);
			
			var hash = hash_string(wd);
			
			if (total_word_map[hash])
			{
				total_word_map[hash].push(wd);
			}
			else
			{
				total_word_map[hash] = [wd];
			}
		}
	}
	
	// Store this word list in RAM.
	total_word_lists[word_len] = aWords;
}


// gen_rand - generate a random number from [0, n_max-1].
function gen_rand(n_max)
{
	return Math.floor(Math.random()*n_max);
}


// scramble_word - create a random anagram of a word.
function scramble_word(wd)
{
	var aChars = wd.toUpperCase().split("");
	var new_wd = "";
	
	for (var len = aChars.length; len>0; len--)
	{
		var x = gen_rand(len);
		new_wd += aChars.splice(x,1);
	}
	
	return new_wd;
}



