/*
	Word Twiddle
	JavaScript game engine
	Version 0.9
	Copyright 2011 by Andrew Krywaniuk
	
	License is hereby granted to freely distribute this game for personal and non-commercial use.
	For other uses, please contact the author.
*/


var g_current_word = null;

// Create both anagram map & array for optimized access.
var g_anagram_map = new Object();
var g_anagram_list = [];
// The letter map for the current word.
var g_letter_map = new Object();

// g_game_stats - run-time state for individual words in the current game.
var g_game_stats = [];


// Configuration options.
var g_bingo_length = 6;
var g_round_time = 60;

// Run-time status & timer.
var g_game_in_progress = false;
var g_time_remaining = 0;
var g_game_timer = null;



// Words are scored according to their length.
var score_vals = [0,0,0,1,2,5,8,13];
var bingo_bonus = 25;


// choose_word - choose a word from the global list
// TODO: this algorithm is more likely to choose words having multiple anagrams, which is not
// not necessarily what we want.
function choose_word(word_len)
{
	var olist = total_word_lists[word_len];
	if (!olist) return null;
	
	var len = olist.length;
	var r_num = gen_rand(len);
	
	return olist[r_num];
}


// NumberOfSetBits - got this off the web
// it probably has more precision than we really need.
function NumberOfSetBits(i)
{
    i = i - ((i >> 1) & 0x55555555);
    i = (i & 0x33333333) + ((i >> 2) & 0x33333333);
    return ((i + (i >> 4) & 0x0F0F0F0F) * 0x01010101) >> 24;
}


// create_letter_map - create a map containing a count of all the letters in the source word.
function create_letter_map(wd)
{
	var lmap = new Object();
	var len = wd.length;
	
	for (var i=0; i<len; i++)
	{
		var ch = wd[i];
		if (lmap[ch])
		{
			lmap[ch]++;
		}
		else
		{
			lmap[ch] = 1;
		}
	}
	
	return lmap;
}


// confirm_subword - confirm that a word is a sub-anagram of another word by confirming that
// the letter map is a subset of the source word's letter map.
function confirm_subword(test_wd, src_letter_map)
{
	var test_letter_map = create_letter_map(test_wd);
	var ch;
	
	for (ch in test_letter_map)
	{
		var tval = test_letter_map[ch];
		var sval = src_letter_map[ch];
		if ((!sval) || (sval < tval)) return false;
	}
	
	return true;
}
				

// calc_all_anagrams - this function calculates all anagrams (3+ chars)
// of a given word. Store them in a map to eliminate duplicates.
function calc_all_anagrams(src_wd)
{
	var lt_hash = [];
	var wd_map = new Object();
	
	var n = src_wd.length;
	var i_max = (1 << n);
	var i, j;
	
	var src_letter_map = create_letter_map(src_wd);
	
	// Cache the hash values for each component letter.
	for (i=0; i<n; i++)
	{
		lt_hash[i] = hash_string(src_wd.substr(i,1));
	}
	
	// Enumerate all bitmaps up to 2^N with at least 3 bits set
	for (i=0; i<i_max; i++)
	{
		// Prune out the sets with less than 3 components
		if (NumberOfSetBits(i) < 3) continue;
		
		// Recalculate the hash for this subset
		var hash = 0;
		
		for (j=0; j<n; j++)
		{
			if ((1 << j) & i)
			{
				hash += lt_hash[j];
			}
		}
		
		// Check if this hash exists in the global map.
		var aMatches = total_word_map[hash];
		if (aMatches)
		{
			var aLen = aMatches.length;
			for (j=0; j<aLen; j++)
			{
				var name = aMatches[j];
				
				// Check for spurious matches (which would imply that our hash function is not perfect).
				if (!confirm_subword(name, src_letter_map))
				{
					var match_letters = "";
					for (j=0; j<n; j++)
					{
						if ((1 << j) & i)
						{
							match_letters += src_wd[j];
						}
					}
					
					dbg_msg("Spurious anagram: " + name + " vs. " + src_wd + ", hash=" + hash + ", letters: " + match_letters);
					continue;
				}
					
				// Set the matches to '0', meaning the user hasn't found them yet.
				wd_map[name] = 0;
			}
		}
	}
		
	
	wd_map[src_wd] = 0;
	return wd_map;
}


// dump_all_anagrams - dump all anagrams to the debug console.
function dump_all_anagrams()
{
	dbg_msg("anagrams: ");
	
/*	var len = g_anagram_list.length;
	for (var i=0; i<len; i++)
	{
		dbg_msg(g_anagram_list[i]);
	} */
	
	dbg_msg(g_anagram_list);
}


// lookup_word - check if a word exists in the dictionary using the word map.
function lookup_word(wd)
{
	var hash = hash_string(wd);
dbg_msg("word = " + wd + ", hash = " + hash);


	var exists = false;
	var aEntry = total_word_map[hash];
	if (aEntry)
	{
		var i;
		for (i=0; i< aEntry.length; i++)
		{
			if (aEntry[i] == wd)
			{
				exists = true;
				break;
			}
		}
	}
	
	dbg_msg("exists = " + exists);
	return exists;
}


// handler for the lookup word test button.
function on_click_lookup_word()
{
	var wd = document.getElementById("eb_lookup_word").value;
	lookup_word(wd);
}


// on_click_load_word - handler for the test button (load word)
function on_click_load_word()
{
	var wd = document.getElementById("eb_lookup_word").value;
	change_current_word(wd);
}


// orderWords - comparison function for sorting the anagram list.
function orderWords(a,b)
{
	if (a.length > b.length) return 1;
	if (a.length < b.length) return -1;
	return (a > b) ? 1 : -1;
}


// build_anagram_list - convert the anagram map into a sorted list of words.
function build_anagram_list(src_map)
{
	var wd_list = [];
	
	for (name in src_map)
	{
		wd_list.push(name);
	}

	wd_list.sort(orderWords);
	
	// Now re-index the anagram map to refer to the array indexes.
	var len = wd_list.length;
	for (var i=0; i<len; i++)
	{
		var name = wd_list[i];
		src_map[name] = i;
	}
	
	return wd_list;
}


// process_guess - process a guess from the user.
function process_guess(wd)
{
	var idx = g_anagram_map[wd];
	if (idx == null) return false;
	
	// Watch out for duplicates.
	if (!g_game_stats[idx].found)
	{
		g_game_stats[idx].found = true;
		g_anagrams_found++;

		var len = wd.length;
		g_running_score += score_vals[len];
		if (len == g_bingo_length)
			g_running_score += bingo_bonus;
		
		reveal_word(idx);
		update_game_stats();
		
		if (g_anagrams_found == g_total_anagrams)
		{
dbg_msg("Found all words");
			setTimeout(end_round, 1);
		}
	}
	
	return true;
}


function change_current_word(wd)
{
	g_current_word = wd;
	g_anagram_map = calc_all_anagrams(wd);
	g_letter_map = create_letter_map(wd);
	g_anagram_list = build_anagram_list(g_anagram_map);
}


function init_game_stats()
{
	g_total_anagrams = g_anagram_list.length;
	g_anagrams_found = 0;
	g_running_score = 0;
	
	g_game_stats = [];
	
	for (var i=0; i<g_total_anagrams; i++)
	{
		g_game_stats[i] = new Object();
		g_game_stats[i].found = 0;
	}
}


// timer_tick - called every second to time the game.
function timer_tick()
{
	g_time_remaining--;
	update_timer_display();
	
	if (g_time_remaining == 0)
	{
		g_game_timer = null;
		end_round();
	}
	else
	{
		g_game_timer = window.setTimeout(timer_tick, 1000);
	}
}


// start_game_timer - begin the countdown timer that times the rounds.
function start_game_timer()
{
	g_game_in_progress = true;
	g_time_remaining = g_round_time;
	g_game_timer = window.setTimeout(timer_tick, 1000);
	
	update_game_controls();
	
	document.getElementById("eb_input_word").focus();
}


// start_game - setup a new game.
function start_game()
{
	var wd = choose_word(g_bingo_length);
	change_current_word(wd);
	
	init_game_stats();
	update_game_stats();	
	layout_words();
	
	var sp_swd = document.getElementById("sp_scrambled_word");
	g_scrambled_word = scramble_word(g_current_word);
	sp_swd.innerHTML = g_scrambled_word;
	
	setTimeout(start_game_timer, 1);
}


// setup_default_word_display - start with a default word to make the GUI look better.
function setup_default_word_display()
{
	var wd = "kavass"; // "matter";
	change_current_word(wd);
	// Reset this to 0, otherwise it looks silly.
	g_total_anagrams = 0;
	init_game_stats();
	update_game_stats();	
	layout_words();
}

